include 'stdsage.pxi'

cdef class Elt:
    cdef long k


def bench_pynew(long n):
    cdef long m
    cdef Elt x
    for m in range(n):
        x = PY_NEW(Elt)
        x.k = m
    
def bench_long(long n):
    cdef long m, k = 0
    for m in range(n):
        k = k + m
    return k

def bench_malloc(long n):
    cdef long m
    cdef long* x
    for m in range(n):
        x = <long*> sage_malloc(sizeof(long))

def bench_new_object(long n):
    cdef long m
    cdef Elt x
    for m in range(n):
        x = Elt()
        x.k = m
    

from sage.rings.integer cimport Integer
def bench_integer(long n):
    cdef long m
    cdef Integer x
    for m in range(n):
        x = PY_NEW(Integer)

### Figure out how to use C++ support to get operator overloading with C++ class's...?
        
## cdef extern from "bench_cpp.cpp":
##     cdef cppclass EltCPP:
##         long x
##     cdef void bench_cpp(long n)
##     EltCPP operator+(EltCPP, EltCPP)

## def bench_cpp_new(long n):
##     cdef EltCPP* x
##     for m in range(n):
##         x = new EltCPP()
##         del x
    
## def bench_cpp_stack(long n):
##     cdef EltCPP x, y, z
##     y = EltCPP(); z = EltCPP();
##     cdef long m
##     for m in range(n):
##         y = y + z + z + y
##     return y.x

    
    
cdef extern from "bench_cpp.cpp":
    cdef cppclass ResidueElement:
        long element[2]
        long moduli[2]
        ResidueElement()
    ResidueElement operator+(ResidueElement, ResidueElement)
    ResidueElement operator*(ResidueElement, ResidueElement)    

def bench_cpp_reselt(long n):
    cdef ResidueElement y, z
    y = ResidueElement()
    z = ResidueElement()
    y.moduli[0] = 11
    y.moduli[1] = 11
    cdef long m
    for m in range(n):
        y = y + z
    return y.element[0]

cdef struct res_elt:
    long element[2]
    long moduli[2]

cdef res_elt res_elt_add(res_elt x, res_elt y):
    cdef int i
    cdef res_elt z
    for i in range(2):
        z.element[i] = (x.element[i] + y.element[i])%x.moduli[i]
        z.moduli[i] = x.moduli[i]
    return z

def bench_cython_reselt_direct(long n):
    cdef res_elt y, z
    y.moduli[0] = 11
    y.moduli[1] = 11
    cdef long m
    for m in range(n):
        y = res_elt_add(y, z)
    return y.element[0]

def bench_cython_reselt_direct2(long n):
    cdef res_elt y, z
    y.moduli[0] = 11
    y.moduli[1] = 11
    cdef long m
    cdef int i
    for m in range(n):
        for i in range(2):
            y.element[i] = (y.element[i] + z.element[i])%y.moduli[i]
            y.moduli[i] = z.moduli[i]
        
    return y.element[0]

ctypedef long residue_element[2]

def bench_cython_reselt_direct3(long n):
    cdef residue_element y, z
    cdef long m
    cdef int i
    for m in range(n):
        for i in range(2):
            y[i] = (y[i] + z[i])%11
    return y[0]


cdef extern from "bench_cpp.cpp":
    cdef cppclass element:
        long x[2]
    element operator+(element, element)


def bench_typedef_op_overload(long n):
    cdef element x, y
    cdef long m
    cdef int i
    x.x[0] = 0
    y.x[0] = 1
    for m in range(n):
        x = x + y
    return x.x[0]

# cdef cppclass element2:
#      long elt[2]
# cdef element2 operator+(element2 x, element2 y):
#      cdef element2 z
#      z.elt[0] = (x.elt[0] + y.elt[0]) % 11
#      z.elt[1] = (x.elt[1] + y.elt[1]) % 11
#      return z

